﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Threading;
using System.Diagnostics;

namespace HMI2
{
    public partial class ApplicationWindow : Form
    {

        byte[] Txbuff1 = new byte[8] { 0x01, 0x04, 0x00, 0x00, 0x00, 0x08, 0xF1, 0xCC };            // MODBUS Read I/P Register for Analog Data of AI-08 unit
        byte[] Txbuff2 = new byte[8] { 0x02, 0x02, 0x00, 0x0C, 0x00, 0x04, 0xB9, 0xF9 };            // MODBUS Read last 4 Digital I/Ps of DI-16 unit
        byte[] Txbuff_DO1_ON = new byte[8] { 0x03, 0x05, 0x00, 0x00, 0xFF, 0x00, 0x8D, 0xD8 };      // MODBUS Write Coil Logic "1" to coil 1 of DO-16 unit
        byte[] Txbuff_DO2_ON = new byte[8] { 0x03, 0x05, 0x00, 0x01, 0xFF, 0x00, 0xDC, 0x18 };      // MODBUS Write Coil Logic "1" to coil 2 of DO-16 unit
        byte[] Txbuff_DO3_ON = new byte[8] { 0x03, 0x05, 0x00, 0x02, 0xFF, 0x00, 0x2C, 0x18 };      // MODBUS Write Coil Logic "1" to coil 3 of DO-16 unit
        byte[] Txbuff_DO4_ON = new byte[8] { 0x03, 0x05, 0x00, 0x03, 0xFF, 0x00, 0x7D, 0xD8 };      // MODBUS Write Coil Logic "1" to coil 4 of DO-16 unit

        byte[] Txbuff_DO1_OFF = new byte[8] { 0x03, 0x05, 0x00, 0x00, 0x00, 0x00, 0xCC, 0x28 };     // MODBUS Write Coil Logic "0" to coil 1 of DO-16 unit
        byte[] Txbuff_DO2_OFF = new byte[8] { 0x03, 0x05, 0x00, 0x01, 0x00, 0x00, 0x9D, 0xE8 };     // MODBUS Write Coil Logic "0" to coil 2 of DO-16 unit
        byte[] Txbuff_DO3_OFF = new byte[8] { 0x03, 0x05, 0x00, 0x02, 0x00, 0x00, 0x6D, 0xE8 };     // MODBUS Write Coil Logic "0" to coil 3 of DO-16 unit
        byte[] Txbuff_DO4_OFF = new byte[8] { 0x03, 0x05, 0x00, 0x03, 0x00, 0x00, 0x3C, 0x28 };     // MODBUS Write Coil Logic "0" to coil 4 of DO-16 unit
        byte[] Txbuff_DO_STATUS = new byte[8] { 0x03, 0x04, 0x00, 0x54, 0x00, 0x01, 0x71, 0xF8 };   // MODBUS Read I/P Register of DO-16 unit to get the Coil status

        int HMI_FLAG = 1;       // Flag to scan AI, DI and DO units sequentially
        int Recieved = 0;       // Recieve buffer from AI-08 unit
        int Recieved1 = 0;      // Recieve buffer from DI-16 unit
        int Recieved2 = 0;      // Recieve buffer from DO-16 unit
        int Counter = 0;


        byte[] Rxbuff1 = new byte[21];
        byte[] Rxbuff2 = new byte[6];
        byte[] Rxbuff3 = new byte[7];
        UInt32 Guage_Value = 0;             // Initial Analog Guage Value

        IntPtr PWM_handle = IntPtr.Zero;
        UInt32 Freq = 4000;                 // PWM Frequency in Hz for Buzzer
        UInt32 Duty = 32768;                // PWM Duty Cycle 

        bool guageDirection = false;        // Guage Direction control variable
        
        public ApplicationWindow()
           
        {
            InitializeComponent();
            serialPort1.Open();
            PWM_handle = pwm.Pwm_Init("PWM4");      // PWM4 is used for Buzzer
        }

        private void button_Exit_Click(object sender, EventArgs e)
        {
            pwm.Pwm_Deinit(PWM_handle);
            timer_Scan.Enabled = false;
            serialPort1.DiscardInBuffer();
            serialPort1.DiscardOutBuffer();
            serialPort1.Close();
            this.Close();
        }

        private void button_Scan_Click(object sender, EventArgs e)
        {

                if (serialPort1.IsOpen == false)
                {
                    MessageBox.Show("Error Opening Port");
                }
                else
                {
                    if (HMI_FLAG == 1)
                    {
                        serialPort1.DiscardInBuffer();
                        serialPort1.Write(Txbuff1, 0, 8);
                        while ((Recieved != 21) && (Counter < 10000))
                        {
                            Counter++;
                            Recieved = serialPort1.BytesToRead;
                        }
                        Debug.Write(Counter);
                        Counter = 0;
                        if (Recieved == 21)
                        {
                            label1.Text = null;
                            serialPort1.Read(Rxbuff1, 0, 21);
                            Recieved = 0;
                        }
                        else
                        {
                            label1.Text = ("AI_COM Error");
                            serialPort1.DiscardInBuffer();

                        }

                        Guage_Value = Rxbuff1[3];
                        Guage_Value = ((Guage_Value) * 256);
                        Guage_Value = ((Guage_Value) | (Rxbuff1[4]));
                        if (guageDirection == false)
                        {
                            AI_gauge1.Value = ((int)(Guage_Value / 100));
                        }
                        else
                            AI_gauge1.Value = (50 - ((int)(Guage_Value / 100)));

                        
                        Guage_Value = Rxbuff1[5];
                        Guage_Value = ((Guage_Value) * 256);
                        Guage_Value = ((Guage_Value) | (Rxbuff1[6]));
                        if (guageDirection == false)
                        {
                            AI_gauge2.Value = ((int)(Guage_Value / 100));
                        }
                        else
                            AI_gauge2.Value = (50 - ((int)(Guage_Value / 100)));

                        
                        Guage_Value = Rxbuff1[7];
                        Guage_Value = ((Guage_Value) * 256);
                        Guage_Value = ((Guage_Value) | (Rxbuff1[8]));
                        if (guageDirection == false)
                        {
                            AI_gauge3.Value = ((int)(Guage_Value / 100));
                        }
                        else
                            AI_gauge3.Value = (50 - ((int)(Guage_Value / 100)));


                        Guage_Value = Rxbuff1[9];
                        Guage_Value = ((Guage_Value) * 256);
                        Guage_Value = ((Guage_Value) | (Rxbuff1[10]));
                        if (guageDirection == false)
                        {
                            AI_gauge4.Value = ((int)(Guage_Value / 100));
                        }
                        else
                            AI_gauge4.Value = (50 - ((int)(Guage_Value / 100)));


                        HMI_FLAG = 2;
                    }

    //--------------------------------------------------------------------------------------// 

                    else if (HMI_FLAG == 2)
                    {

                        serialPort1.DiscardInBuffer();
                        serialPort1.Write(Txbuff2, 0, 8);
                        while ((Recieved1 != 6) && (Counter < 10000))
                        {
                            Counter++;
                            Recieved1 = serialPort1.BytesToRead;
                        }
                        Counter = 0;
                        if (Recieved1 == 6)
                        {
                            label15.Text = null;
                            serialPort1.Read(Rxbuff2, 0, 6);
                            Recieved1 = 0;
                        }
                        else
                        {
                            label15.Text = ("DI_COM Error");
                            serialPort1.DiscardInBuffer();
                        }
                        switch (Rxbuff2[3])
                        {
                            case 0x01:

                                GreenLed1.Visible = true;
                                GreenLed2.Visible = false;
                                GreenLed3.Visible = false;
                                GreenLed4.Visible = false;
                                break;

                            case 0x02:

                                GreenLed1.Visible = false;
                                GreenLed2.Visible = true;
                                GreenLed3.Visible = false;
                                GreenLed4.Visible = false;
                                break;

                            case 0x03:

                                GreenLed1.Visible = true;
                                GreenLed2.Visible = true;
                                GreenLed3.Visible = false;
                                GreenLed4.Visible = false;
                                break;

                            case 0x04:

                                GreenLed1.Visible = false;
                                GreenLed2.Visible = false;
                                GreenLed3.Visible = true;
                                GreenLed4.Visible = false;
                                break;

                            case 0x05:

                                GreenLed1.Visible = true;
                                GreenLed2.Visible = false;
                                GreenLed3.Visible = true;
                                GreenLed4.Visible = false;
                                break;

                            case 0x06:

                                GreenLed1.Visible = false;
                                GreenLed2.Visible = true;
                                GreenLed3.Visible = true;
                                GreenLed4.Visible = false;
                                break;

                            case 0x07:

                                GreenLed1.Visible = true;
                                GreenLed2.Visible = true;
                                GreenLed3.Visible = true;
                                GreenLed4.Visible = false;
                                break;

                            case 0x08:

                                GreenLed1.Visible = false;
                                GreenLed2.Visible = false;
                                GreenLed3.Visible = false;
                                GreenLed4.Visible = true;
                                break;

                            case 0x09:

                                GreenLed1.Visible = true;
                                GreenLed2.Visible = false;
                                GreenLed3.Visible = false;
                                GreenLed4.Visible = true;
                                break;

                            case 0x0A:

                                GreenLed1.Visible = false;
                                GreenLed2.Visible = true;
                                GreenLed3.Visible = false;
                                GreenLed4.Visible = true;
                                break;

                            case 0x0B:

                                GreenLed1.Visible = true;
                                GreenLed2.Visible = true;
                                GreenLed3.Visible = false;
                                GreenLed4.Visible = true;
                                break;

                            case 0x0C:

                                GreenLed1.Visible = false;
                                GreenLed2.Visible = false;
                                GreenLed3.Visible = true;
                                GreenLed4.Visible = true;
                                break;

                            case 0x0D:

                                GreenLed1.Visible = true;
                                GreenLed2.Visible = false;
                                GreenLed3.Visible = true;
                                GreenLed4.Visible = true;
                                break;

                            case 0x0E:

                                GreenLed1.Visible = false;
                                GreenLed2.Visible = true;
                                GreenLed3.Visible = true;
                                GreenLed4.Visible = true;
                                break;

                            case 0x0F:

                                GreenLed1.Visible = true;
                                GreenLed2.Visible = true;
                                GreenLed3.Visible = true;
                                GreenLed4.Visible = true;
                                break;

                            default:

                                GreenLed1.Visible = false;
                                GreenLed2.Visible = false;
                                GreenLed3.Visible = false;
                                GreenLed4.Visible = false;
                                break;

                        }
                        HMI_FLAG = 3;
                    }
                    //------------------------------------------------------------------------------------------//

                    else if (HMI_FLAG == 3)
                    {
                        serialPort1.DiscardInBuffer();
                        serialPort1.Write(Txbuff_DO_STATUS, 0, 8);
                        while ((Recieved2 != 7) && (Counter < 10000))
                        {
                            Counter++;
                            Recieved2 = serialPort1.BytesToRead;
                        }
                        Counter = 0;
                        if (Recieved2 == 7)
                        {
                            label16.Text = null;
                            serialPort1.Read(Rxbuff3, 0, 7);
                            Recieved2 = 0;
                        }
                        else
                        {
                            label16.Text = ("DO_COM Error");
                            serialPort1.DiscardInBuffer();

                        }

                        switch (Rxbuff3[4])
                        {
                            case 0x01:

                                
                                button_DO1.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO2.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO3.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO4.BackColor = Color.FromArgb(255, 90, 0);
                                break;

                            case 0x02:

                                button_DO1.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO2.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO3.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO4.BackColor = Color.FromArgb(255, 90, 0);
                                break;

                            case 0x03:

                                button_DO1.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO2.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO3.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO4.BackColor = Color.FromArgb(255, 90, 0);
                                break;

                            case 0x04:

                                button_DO1.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO2.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO3.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO4.BackColor = Color.FromArgb(255, 90, 0);
                                break;

                            case 0x05:

                                button_DO1.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO2.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO3.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO4.BackColor = Color.FromArgb(255, 90, 0);
                                break;

                            case 0x06:

                                button_DO1.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO2.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO3.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO4.BackColor = Color.FromArgb(255, 90, 0);
                                break;

                            case 0x07:

                                button_DO1.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO2.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO3.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO4.BackColor = Color.FromArgb(255, 90, 0);
                                break;

                            case 0x08:

                                button_DO1.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO2.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO3.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO4.BackColor = Color.FromArgb(150, 200, 55);
                                break;

                            case 0x09:

                                button_DO1.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO2.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO3.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO4.BackColor = Color.FromArgb(150, 200, 55);
                                break;

                            case 0x0A:

                                button_DO1.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO2.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO3.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO4.BackColor = Color.FromArgb(150, 200, 55);
                                break;

                            case 0x0B:

                                button_DO1.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO2.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO3.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO4.BackColor = Color.FromArgb(150, 200, 55);
                                break;

                            case 0x0C:

                                button_DO1.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO2.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO3.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO4.BackColor = Color.FromArgb(150, 200, 55);
                                break;

                            case 0x0D:

                                button_DO1.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO2.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO3.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO4.BackColor = Color.FromArgb(150, 200, 55);
                                break;

                            case 0x0E:

                                button_DO1.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO2.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO3.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO4.BackColor = Color.FromArgb(150, 200, 55);
                                break;

                            case 0x0F:

                                button_DO1.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO2.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO3.BackColor = Color.FromArgb(150, 200, 55);
                                button_DO4.BackColor = Color.FromArgb(150, 200, 55);
                                break;

                            default:

                                button_DO1.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO2.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO3.BackColor = Color.FromArgb(255, 90, 0);
                                button_DO4.BackColor = Color.FromArgb(255, 90, 0);
                                break;

                        }
                        HMI_FLAG = 1;
                    }


                
            }
        }

        private void button5_Click(object sender, EventArgs e)
        {
            button_DO1.Enabled = false;
            timer_Scan.Enabled = false;
            timer3.Enabled = true;
        }

        private void button6_Click(object sender, EventArgs e)
        {
            button_DO2.Enabled = false;
            timer_Scan.Enabled = false;
            timer4.Enabled = true;
        }

        private void button7_Click(object sender, EventArgs e)
        {
            button_DO3.Enabled = false;
            timer_Scan.Enabled = false;
            timer5.Enabled = true;
        }

        private void button8_Click(object sender, EventArgs e)
        {
            button_DO4.Enabled = false;
            timer_Scan.Enabled = false;
            timer6.Enabled = true;
        }

        private void timer2_Tick(object sender, EventArgs e)
        {
            pwm.Pwm_Close(PWM_handle);
            timer2.Enabled = false;
            timer_Scan.Enabled = true;
            button_DO1.Enabled = true;
            button_DO2.Enabled = true;
            button_DO3.Enabled = true;
            button_DO4.Enabled = true;
        }

        private void timer3_Tick(object sender, EventArgs e)
        {
            timer3.Enabled = false;

            if (button_DO1.BackColor == Color.FromArgb(255, 90, 0))
            {
                serialPort1.Write(Txbuff_DO1_ON, 0, 8);
            }
            else
            {
                serialPort1.Write(Txbuff_DO1_OFF, 0, 8);
            }
            pwm.Pwm_Open(PWM_handle);
            pwm.Pwm_SetPwm(PWM_handle, Freq, Duty);
            timer2.Enabled = true;
        }

        private void timer4_Tick(object sender, EventArgs e)
        {
            timer4.Enabled = false;

            if (button_DO2.BackColor == Color.FromArgb(255, 90, 0))
            {
                serialPort1.Write(Txbuff_DO2_ON, 0, 8);
            }
            else
            {
                serialPort1.Write(Txbuff_DO2_OFF, 0, 8);
            }
            pwm.Pwm_Open(PWM_handle);
            pwm.Pwm_SetPwm(PWM_handle, Freq, Duty);
            timer2.Enabled = true;

        }

        private void timer5_Tick(object sender, EventArgs e)
        {
            timer5.Enabled = false;

            if (button_DO3.BackColor == Color.FromArgb(255, 90, 0))
            {
                serialPort1.Write(Txbuff_DO3_ON, 0, 8);
            }
            else
            {
                serialPort1.Write(Txbuff_DO3_OFF, 0, 8);
            }
            pwm.Pwm_Open(PWM_handle);
            pwm.Pwm_SetPwm(PWM_handle, Freq, Duty);
            timer2.Enabled = true;
        }

        private void timer6_Tick(object sender, EventArgs e)
        {
            timer6.Enabled = false;

            if (button_DO4.BackColor == Color.FromArgb(255, 90, 0))
            {
                serialPort1.Write(Txbuff_DO4_ON, 0, 8);
            }
            else
            {
                serialPort1.Write(Txbuff_DO4_OFF, 0, 8);
            }
            pwm.Pwm_Open(PWM_handle);
            pwm.Pwm_SetPwm(PWM_handle, Freq, Duty);
            timer2.Enabled = true;
        }

        private void GreenLed2_Click(object sender, EventArgs e)
        {
            timer_Scan.Enabled = false;
            guageDirection ^= true;
            timer_Scan.Enabled = true;
        }

    }
}