﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;


namespace HMI2
{

    public partial class TestWindow : Form
    {

        string Recived;                     // MOBBUS reciev buffer1
        string Recived1;                    // MODBUS reciev buffer2
        string Recived2;                    // RS232 recieve buffer
        bool Flag1, Flag2 = false;          // MODBUS communication direction control flags 
        UInt32 Freq = 4000;                 // PWM Frequency in Hz
        UInt32 Duty = 32768;                // PWM Duty Cycle
        UInt32 EraseCheck = 0;              // Variable for SPI Flash communication check
        UInt32 ReadWriteCheck = 0;          // Variable for SPI Flash communication check

        UInt32[] rxBuffer = new UInt32[300]; // SPI Receive buffer
        UInt32 ReadVal = 0;

        IntPtr SPI_handle = IntPtr.Zero;    // SPI Handle
        IntPtr GPIO_handle = IntPtr.Zero;   //GPIO Handle
        IntPtr PWM_handle = IntPtr.Zero;    // PWM Handle


        const ushort ioColibriPin = 0x20;
        ushort pinNumber96 = 96;
        ushort pinNumber81 = 81;
        ushort pinNumber94 = 94;
        ushort pinNumber75 = 75;
        ushort pinNumber65 = 65;
        ushort pinNumber98 = 98;
        ushort pinNumber101 = 101;
        ushort pinNumber97 = 97;
        ushort pinNumber67 = 67;
        ushort pinNumber34 = 34;
        ushort pinNumber135 = 135;
        ushort pinNumber103 = 103;
        ushort pinNumber133 = 133;

        gpio.uIo io96;
        gpio.uIo io81;
        gpio.uIo io94;
        gpio.uIo io75;
        gpio.uIo io65;
        gpio.uIo io98;
        gpio.uIo io101;
        gpio.uIo io97;
        gpio.uIo io67;
        gpio.uIo io34;
        gpio.uIo io135;
        gpio.uIo io103;
        gpio.uIo io133;


        public TestWindow()
        {
            InitializeComponent();

            PWM_handle = pwm.Pwm_Init("PWM4");  // For Buzzer PWM_D is used


            SPI_handle = spi.Spi_Init("SPI2");  // SPI2 is used 
            spi.Spi_SetConfigInt(SPI_handle, "BitRateHz", 1000 * 1000, TdxCommon.ParamStorageType.StoreVolatile);

            GPIO_handle = gpio.Gpio_Init(null);
            gpio.Gpio_Open(GPIO_handle);
            io96 = new gpio.uIo(pinNumber96, ioColibriPin);
            io81 = new gpio.uIo(pinNumber81, ioColibriPin);
            io94 = new gpio.uIo(pinNumber94, ioColibriPin);
            io75 = new gpio.uIo(pinNumber75, ioColibriPin);
            io65 = new gpio.uIo(pinNumber65, ioColibriPin);
            io98 = new gpio.uIo(pinNumber98, ioColibriPin);
            io101 = new gpio.uIo(pinNumber101, ioColibriPin);
            io97 = new gpio.uIo(pinNumber97, ioColibriPin);
            io67 = new gpio.uIo(pinNumber67, ioColibriPin);
            io34 = new gpio.uIo(pinNumber34, ioColibriPin);
            io135 = new gpio.uIo(pinNumber135, ioColibriPin);
            io103 = new gpio.uIo(pinNumber103, ioColibriPin);
            io133 = new gpio.uIo(pinNumber133, ioColibriPin);

            gpio.Gpio_ConfigureAsGpio(GPIO_handle, io96);
            gpio.Gpio_ConfigureAsGpio(GPIO_handle, io81);
            gpio.Gpio_ConfigureAsGpio(GPIO_handle, io94);
            gpio.Gpio_ConfigureAsGpio(GPIO_handle, io75);
            gpio.Gpio_ConfigureAsGpio(GPIO_handle, io65);
            gpio.Gpio_ConfigureAsGpio(GPIO_handle, io98);
            gpio.Gpio_ConfigureAsGpio(GPIO_handle, io101);
            gpio.Gpio_ConfigureAsGpio(GPIO_handle, io97);
            gpio.Gpio_ConfigureAsGpio(GPIO_handle, io34);
            gpio.Gpio_ConfigureAsGpio(GPIO_handle, io135);
            gpio.Gpio_ConfigureAsGpio(GPIO_handle, io103);
            gpio.Gpio_ConfigureAsGpio(GPIO_handle, io133);

            gpio.Gpio_SetDir(GPIO_handle, io96, gpio.tIoDirection.ioInput);     // GPIO 1
            gpio.Gpio_SetDir(GPIO_handle, io81, gpio.tIoDirection.ioInput);     // GPIO 2
            gpio.Gpio_SetDir(GPIO_handle, io94, gpio.tIoDirection.ioInput);     // GPIO 3
            gpio.Gpio_SetDir(GPIO_handle, io75, gpio.tIoDirection.ioInput);     // GPIO 4
            gpio.Gpio_SetDir(GPIO_handle, io65, gpio.tIoDirection.ioInput);     // GPIO 5
            gpio.Gpio_SetDir(GPIO_handle, io98, gpio.tIoDirection.ioInput);     // GPIO 6
            gpio.Gpio_SetDir(GPIO_handle, io101, gpio.tIoDirection.ioInput);    // GPIO 7
            gpio.Gpio_SetDir(GPIO_handle, io97, gpio.tIoDirection.ioInput);     // GPIO 8
            gpio.Gpio_SetDir(GPIO_handle, io67, gpio.tIoDirection.ioOutput);    // PWM_D O/P
            gpio.Gpio_SetDir(GPIO_handle, io34, gpio.tIoDirection.ioOutput);    // UART_B RTS
            gpio.Gpio_SetDir(GPIO_handle, io135, gpio.tIoDirection.ioOutput);   // UART_C RTS
            gpio.Gpio_SetDir(GPIO_handle, io103, gpio.tIoDirection.ioOutput);   // SPI Pin "WP#"
            gpio.Gpio_SetDir(GPIO_handle, io133, gpio.tIoDirection.ioOutput);   // SPI Pin "HOLD"

            gpio.Gpio_SetLevel(GPIO_handle, io103, gpio.tIoLevel.ioHigh);       // Pull up SPI Pin "WP#"
            gpio.Gpio_SetLevel(GPIO_handle, io133, gpio.tIoLevel.ioHigh);       // Pull up SPI Pin "HOLD"

        }


        private void Scan_Click(object sender, EventArgs e)     // Scan GPIOs for their I/P levels and change the corresponding color accordingly
        {


            if (gpio.Gpio_GetLevel(GPIO_handle, io96) == gpio.tIoLevel.ioHigh)
                button_LedGPIO1.BackColor = Color.FromArgb(150, 200, 55);
            else
                button_LedGPIO1.BackColor = Color.FromArgb(255, 90, 0);


            if (gpio.Gpio_GetLevel(GPIO_handle, io81) == gpio.tIoLevel.ioHigh)
                button_LedGPIO2.BackColor = Color.FromArgb(150, 200, 55);
            else
                button_LedGPIO2.BackColor = Color.FromArgb(255, 90, 0);


            if (gpio.Gpio_GetLevel(GPIO_handle, io94) == gpio.tIoLevel.ioHigh)
                button_LedGPIO3.BackColor = Color.FromArgb(150, 200, 55);
            else
                button_LedGPIO3.BackColor = Color.FromArgb(255, 90, 0);


            if (gpio.Gpio_GetLevel(GPIO_handle, io75) == gpio.tIoLevel.ioHigh)
                button_LedGPIO4.BackColor = Color.FromArgb(150, 200, 55);
            else
                button_LedGPIO4.BackColor = Color.FromArgb(255, 90, 0);


            if (gpio.Gpio_GetLevel(GPIO_handle, io65) == gpio.tIoLevel.ioHigh)
                button_LedGPIO5.BackColor = Color.FromArgb(150, 200, 55);
            else
                button_LedGPIO5.BackColor = Color.FromArgb(255, 90, 0);


            if (gpio.Gpio_GetLevel(GPIO_handle, io98) == gpio.tIoLevel.ioHigh)
                button_LedGPIO6.BackColor = Color.FromArgb(150, 200, 55);
            else
                button_LedGPIO6.BackColor = Color.FromArgb(255, 90, 0);


            if (gpio.Gpio_GetLevel(GPIO_handle, io101) == gpio.tIoLevel.ioHigh)
                button_LedGPIO7.BackColor = Color.FromArgb(150, 200, 55);
            else
                button_LedGPIO7.BackColor = Color.FromArgb(255, 90, 0);


            if (gpio.Gpio_GetLevel(GPIO_handle, io97) == gpio.tIoLevel.ioHigh)
                button_LedGPIO8.BackColor = Color.FromArgb(150, 200, 55);
            else
                button_LedGPIO8.BackColor = Color.FromArgb(255, 90, 0);


        }

        private void Buzzer_Click(object sender, EventArgs e)           // Sound buzzer by applying 4KHz PWM I/P
        {
            pwm.Pwm_Open(PWM_handle);
            pwm.Pwm_SetPwm(PWM_handle, Freq, Duty);
            timer_PWMBuzzer.Enabled = true;
        }



        private void checkBox_Auto_Manual_CheckStateChanged(object sender, EventArgs e)  //GPIO Auto or Manual scan slection
        {
            if (checkBox_Auto_Manual.Checked == true)
            {
                Scan.Visible = false;
                checkBox_Auto_Manual.Text = "Auto";
                timer_GpioScan.Enabled = true;
            }
            else
            {
                Scan.Visible = true;
                checkBox_Auto_Manual.Text = "Manual";
                timer_GpioScan.Enabled = false;
            }
        }


        private void timer_PWMBuzzer_Tick(object sender, EventArgs e)       // Stop buzzer after timer_PWMBuzzer runs over
        {
            pwm.Pwm_Close(PWM_handle);
            timer_PWMBuzzer.Enabled = false;
        }

        private void Modbus_Click(object sender, EventArgs e)               // Test MODBUS communication 
        {

            serialPort2.Open();
            serialPort3.Open();

            if ((serialPort2.IsOpen == false) || (serialPort3.IsOpen == false))
            {
                serialPort2.Close();
                serialPort3.Close();
                MessageBox.Show("Error opening ports");
            }
            else
            {
                Modbus.Enabled = false;

                gpio.Gpio_SetLevel(GPIO_handle, io34, gpio.tIoLevel.ioHigh);
                gpio.Gpio_SetLevel(GPIO_handle, io135, gpio.tIoLevel.ioLow);

                serialPort2.WriteLine("hello world, we are Toradex Systems & this is a testing application for Modbus and RS232");
                timer_MOD_X6_X7.Enabled = true;
            }

        }

        private void timer3_Tick(object sender, EventArgs e)            // Check data recieved form connector X6 to connector X7 on HMI Board
        {
            timer_MOD_X6_X7.Enabled = false;
            Recived = serialPort3.ReadExisting();
            if (string.Compare("hello world, we are Toradex Systems & this is a testing application for Modbus and RS232\n", Recived) == 0)
            {
                X6_X7.ForeColor = Color.FromArgb(150, 200, 55);
                Flag1 = true;
            }
            else
            {
                X6_X7.ForeColor = Color.FromArgb(255, 90, 0);
                Flag1 = false;
            }

            gpio.Gpio_SetLevel(GPIO_handle, io34, gpio.tIoLevel.ioLow);
            gpio.Gpio_SetLevel(GPIO_handle, io135, gpio.tIoLevel.ioHigh);

            serialPort3.WriteLine("hello world, we are Toradex Systems & this is a testing application for Modbus and RS232");
            timer_MOD_X7_X6.Enabled = true;
        }

        private void timer4_Tick(object sender, EventArgs e)            // Check data recieved form connector X7 to connector X6 on HMI Board
        {
            timer_MOD_X7_X6.Enabled = false;
            Recived1 = serialPort2.ReadExisting();
            if (string.Compare("hello world, we are Toradex Systems & this is a testing application for Modbus and RS232\n", Recived1) == 0)
            {
                X7_X6.ForeColor = Color.FromArgb(150, 200, 55);
                Flag2 = true;
            }

            else
            {
                X7_X6.ForeColor = Color.FromArgb(255, 90, 0);
                Flag2 = false;
            }

            if ((Flag1 && Flag2) == true)
            {
                label_MODBUS.ForeColor = Color.White;
                label_MODBUS.Text = "Pass";
            }
            else
            {
                label_MODBUS.ForeColor = Color.FromArgb(255, 90, 0);
                label_MODBUS.Text = "Failed";
            }

            Modbus.Enabled = true;
            serialPort3.Close();
            serialPort2.Close();
        }

        private void RS232_Click(object sender, EventArgs e)            // Test RS232 communication
        {
            serialPort1.Open();
            serialPort1.DiscardInBuffer();
            if (serialPort1.IsOpen == false)
                MessageBox.Show("Error Opening Port");
            else
            {
                RS232.Enabled = false;
                serialPort1.WriteLine("hello world, we are Toradex Systems & this is a testing application for Modbus and RS232");
                timer_RS232.Enabled = true;
            }
        }

        private void timer5_Tick(object sender, EventArgs e)            // Check RS232 by loopback data on UART1
        {
            timer_RS232.Enabled = false;
            Recived2 = serialPort1.ReadExisting();
            if (string.Compare("hello world, we are Toradex Systems & this is a testing application for Modbus and RS232\n", Recived2) == 0)
            {
                label232.ForeColor = Color.White;
                label232.Text = "Pass";
            }
            else
            {
                label232.ForeColor = Color.FromArgb(255, 90, 0);
                label232.Text = "Failed";
            }

            serialPort1.Close();
            RS232.Enabled = true;

        }

        private void button_Exit_Click(object sender, EventArgs e)  //Exit button click
        {
            pwm.Pwm_Deinit(PWM_handle);
            spi.Spi_Deinit(SPI_handle);
            gpio.Gpio_Deinit(GPIO_handle);
            timer_GpioScan.Enabled = false;
            timer_PWMBuzzer.Enabled = false;
            timer_MOD_X6_X7.Enabled = false;
            timer_MOD_X7_X6.Enabled = false;
            timer_RS232.Enabled = false;
            serialPort1.Close();
            serialPort2.Close();
            serialPort3.Close();
            this.Close();

        }

        private void SPI_Test_Click(object sender, EventArgs e) // Test SPI based Flash Memory
        {
            SPI_Test.Enabled = false;

            spi.Spi_Open(SPI_handle);

            FlashMemory_GetID();

            if (rxBuffer[1] == 0xbf)
            {
                SPI_label_Manufacturer.Text = "Manufacturer : Microchip";

                if (rxBuffer[3] == 0x8E)
                    SPI_label_Capacity.Text = "Capacity : 8Mbit";
                else if (rxBuffer[3] == 0x43)
                    SPI_label_Capacity.Text = "Capacity : 64Mbit";
                else
                    SPI_label_Capacity.Text = "Capacity : Unknown";


                FlashMemory_EnableWriteStatusRegister();
                FlashMemory_WriteStatusRegister(0x00);

                FlashMemory_WriteEnable();

                FlashMemory_SectorErase_4K();

                FlashMemory_BusyCheck();

                FlashMemory_ReadData(0, 256);

                for (uint i = 4; i <= 259; i++)
                {
                    if (rxBuffer[i] != 0xff)
                    {
                        EraseCheck = EraseCheck + 1;
                    }
                }

                if (EraseCheck == 0)
                {
                    checkBox_Erase.Checked = true;
                }

                for (uint i = 0; i <= 256; i++)
                {
                    FlashMemory_WriteByte(i, i);
                    FlashMemory_BusyCheck();
                }


                FlashMemory_ReadData(0, 256);


                for (uint i = 4; i <= 259; i++)
                {
                    if (rxBuffer[i] != (i - 4))
                    {
                        ReadWriteCheck = ReadWriteCheck + 1;
                    }
                }

                if (ReadWriteCheck == 0)
                {
                    checkBox_Read_Write.Checked = true;
                }

                SPI_Test.Enabled = true;

            }

            else
                SPI_label_Manufacturer.Text = "Manufacturer : Unknown";
        }
    }
}

